/*

					Logic Scope AVR

			Copyrights 2009,  Yoshi Hiroshi


*/

#include <avr/io.h>
#include "logicScope.h"
#include "logCommand.h"
#include "uart.h"
#include "commMsg.h"


unsigned long periodTable[] = {
#if defined (__AVR_ATmega644P__)
	(unsigned long)LOG_SAMPLE_PERIOD_6_6M,
	(unsigned long)LOG_SAMPLE_PERIOD_5M,
#endif
#if defined (__AVR_ATmega128__)
	(unsigned long)LOG_SAMPLE_PERIOD_4M,
#endif
	(unsigned long)LOG_SAMPLE_PERIOD_2M,
	(unsigned long)LOG_SAMPLE_PERIOD_1M,
	(unsigned long)LOG_SAMPLE_PERIOD_500K,
	(unsigned long)LOG_SAMPLE_PERIOD_200K,
	(unsigned long)LOG_SAMPLE_PERIOD_100K,
	(unsigned long)LOG_SAMPLE_PERIOD_50K,
	(unsigned long)LOG_SAMPLE_PERIOD_20K,
	(unsigned long)LOG_SAMPLE_PERIOD_10K,
	(unsigned long)LOG_SAMPLE_PERIOD_5K,
	(unsigned long)LOG_SAMPLE_PERIOD_2K,
	(unsigned long)LOG_SAMPLE_PERIOD_1K,
	0
};

//**********************************************************
//		Send Byte
//**********************************************************
void sendUART_1(unsigned char b1)
{
	sendUART(b1);
}

//**********************************************************
//		Send Integer
//**********************************************************
void sendUART_2(unsigned int b1)
{
	sendUART(b1 >> 8);
	sendUART(b1 & 0xFF);
}

//**********************************************************
//		Send 4 Byte Hex Data
//**********************************************************
void sendUART_4(unsigned char b1, unsigned char b2, unsigned char b3, unsigned char b4)
{
	sendUART(b1);
	sendUART(b2);
	sendUART(b3);
	sendUART(b4);
}

//**********************************************************
//		get Unsigned Long
//**********************************************************
unsigned long getLong(unsigned char* ptr)
{
	union {
		unsigned long val;
		unsigned char str[4];
	} change;
	change.str[0] = ptr[3];
	change.str[1] = ptr[2];
	change.str[2] = ptr[1];
	change.str[3] = ptr[0];
	return change.val;
}

//**********************************************************
//		reply Unknown command or framing error
//**********************************************************
//
//	PACKET:
//		command			0xFF
//		length(higher)	0x00
//		length(lower)	0x02
//		resultCode		0x4F	// "N"o good
//		replyCode		0x??
//
//----------------------------------------------------------
void replyUnknown(unsigned char reply)
{
	sendUART_4(COMMAND_REPLY_UNKNOWN, 0x00, 0x02, 'N');
	sendUART_1(reply);
}

//**********************************************************
//		reply Check
//**********************************************************
//
//	PACKET:
//		command			0x21
//		length(higher)	0x00
//		length(lower)	0x06
//		resultCode		0x4F	// "O"k
//		hardMask		0x??
//		buff size		0x?? 0x??
//		ex. buf size	0x?? 0x??
//
//----------------------------------------------------------
void replyCheck()
{
	sendUART_4(COMMAND_CHECK_REPLY, 0x00, 0x06, 'O');
	sendUART_1(lcb.HardwareMask);	// HARDWARE MASK BIT
	sendUART_2(LOG_DATA_BUFFER_SIZE);	// Buffer Size
#if defined (EXTERNAL_MEMORY_USED)
	sendUART_2(LOG_EXTERNAL_BUFFER_SIZE);	// External Buffer Size
#else
	sendUART_2(0);	// External Buffer Size
#endif
	for(int loop = 0; loop < 5; loop++)
	{
		BUSY_LED_WAIT_TRIGGER_ON;
		for(volatile long wait1 = 0; wait1 < 50000; wait1++);
		BUSY_LED_WAIT_TRIGGER_OFF;
		for(volatile long wait2 = 0; wait2 < 10000; wait2++);
	}
}

//**********************************************************
//		reply start sampling
//**********************************************************
//
//	PACKET:
//		command			0x23
//		length(higher)	0x00
//		length(lower)	0x03
//		resultCode		0x??
//		data size		0x?? 0x??
//
//----------------------------------------------------------
void replyStartSampling(unsigned char reply, unsigned int size)
{
	sendUART_4(COMMAND_START_REPLY, 0x00, 0x03, reply);
	sendUART_2(size);
}

//**********************************************************
//		reply end of sampling
//**********************************************************
//
//	PACKET:
//		command			0x23
//		length(higher)	0x00
//		length(lower)	0x03
//		resultCode		0x45
//		data size		0x?? 0x??
//
//----------------------------------------------------------
void replyEndSampling(unsigned int size)
{
	sendUART_4(COMMAND_START_REPLY, 0x00, 0x03, 'E');
	sendUART_2(size);
}


//**********************************************************
//		Reply Log Condtion
//**********************************************************
//
//	PACKET:
//		command			0x25
//		length(higher)	0x00
//		length(lower)	0x09
//		hardMask		0x??
//		triggerMask		0x??
//		triggerPattern	0x??
//		period_0		0x??
//		period_1		0x??
//		period_2		0x??
//		period_3		0x??
//		buff size		0x?? 0x??
//
//----------------------------------------------------------
void replyLogCondition()
{
	sendUART_4(COMMAND_LOGCOND_REPLY, 0x00, 0x09, lcb.HardwareMask);
	sendUART_1(lcb.TriggerMask);
	sendUART_1(lcb.TriggerPattern);
	sendUART_4(lcb.logPeriod[0], lcb.logPeriod[1], lcb.logPeriod[2], lcb.logPeriod[3]);
	sendUART_2(lcb.buffSize);
}


//**********************************************************
//		Send Log Data
//**********************************************************
//
//	PACKET:
//		command			0x27
//		length(higher)	0x??
//		length(lower)	0x??
//		code			0x01
//		bufTop			0x?? 0x??
//		size			0x?? 0x??
//		(data)			....... (length - 7)byte
//
//----------------------------------------------------------
void replyLogData(unsigned int bufTop, unsigned int size)
{
	unsigned int actualSize = ((bufTop + size) <= lcb.buffSize) ? size : (lcb.buffSize - bufTop);
	sendUART_4(COMMAND_GETDATA_REPLY, ((actualSize + 5) >> 8), ((actualSize + 5) & 0xFF), 0x01);
	sendUART_2(bufTop);
	sendUART_2(actualSize);
	for(int index = 0; index < actualSize; index++)
	{
		sendUART_1(lcb.buff[bufTop + index] & lcb.HardwareMask);
	}
}

//==============================================================================
//
//		Message Handler
//
//==============================================================================


//----------------------------------------------------------
//		Check Command
//	PACKET:
//		command			0x20
//		length(higher)	0x00
//		length(lower)	0x01
//		actionCode		0x01
//
void doCheckCommand()
{
	lcb.commandBuffer[1] = readUART_1();	// get length HIGH
	lcb.commandBuffer[2] = readUART_1();	// get length LOW
	if((lcb.commandBuffer[1] * 256 + lcb.commandBuffer[2]) == 1)
	{
		lcb.commandBuffer[3] = readUART_1();	// get action
		if(lcb.commandBuffer[3] == 0x01)
		{
			replyCheck();
		} else {
			replyUnknown(COMMAND_REPLY_UNKNOWN_CHECK_CODE);
		}
	} else {
		replyUnknown(COMMAND_REPLY_UNKNOWN_CHECK_LENGTH);
	}
}

//----------------------------------------------------------
//		Start Command
//	PACKET:
//		command			0x22
//		length(higher)	0x00
//		length(lower)	0x06
//		period_0
//		period_1
//		period_2
//		period_3
//						150nS 00 00 00 96	/ 250nS 00 00 00 FA / 500nS 00 00 01 F4
//						  1uS 00 00 03 E8	/   2uS 00 00 07 D0	/   5uS 00 00 13 88
//						 10uS 00 00 27 10	/  20uS 00 00 4E 20	/  50uS 00 00 C3 50
//						100uS 00 01 86 A0	/ 200uS 00 03 0D 40 / 500uS 00 07 A1 20
//						  1mS 00 0F 42 40
//		triggerMask		0x00
//		TriggerPattern	0x00
//
void doStartCommand()
{
	lcb.commandBuffer[1] = readUART_1();	// get length HIGH
	lcb.commandBuffer[2] = readUART_1();	// get length LOW
	if(((lcb.commandBuffer[1] << 8) | lcb.commandBuffer[2]) == 6)
	{
		lcb.logPeriod[0] = lcb.commandBuffer[3] = readUART_1();	// get period HIGH 2
		lcb.logPeriod[1] = lcb.commandBuffer[4] = readUART_1();	// get period High 1
		lcb.logPeriod[2] = lcb.commandBuffer[5] = readUART_1();	// get period LOW  1
		lcb.logPeriod[3] = lcb.commandBuffer[6] = readUART_1();	// get period LOW  2
		lcb.period = getLong(&lcb.commandBuffer[3]);
		lcb.TriggerMask  = (readUART_1() & lcb.HardwareMask);
		lcb.TriggerPattern = readUART_1();

		unsigned char index = 0;
		unsigned char periodFound = 0;
		unsigned long eePeriod;
		while((eePeriod = periodTable[index]) != 0)
		{
			if(lcb.period == eePeriod)
			{
				switch(lcb.period)
				{
#if defined (__AVR_ATmega644P__)
					case LOG_SAMPLE_PERIOD_6_6M:		// 150nS  00000096
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_3_Cycle();
						break;
					case LOG_SAMPLE_PERIOD_5M:			// 200nS  000000C8
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_4_Cycle();
						break;
#endif
#if defined (__AVR_ATmega128__)
					case LOG_SAMPLE_PERIOD_4M:			// 250nS  000000FA
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_4_Cycle();
						break;
#endif
					case LOG_SAMPLE_PERIOD_2M:		// 500nS
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_10_Cycle();
						break;
					case LOG_SAMPLE_PERIOD_1M:		// 1uS
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_20_Cycle();
						break;
					case LOG_SAMPLE_PERIOD_500K:	// 2uS
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_40_Cycle();
						break;
					case LOG_SAMPLE_PERIOD_200K:	// 5uS
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_100_Cycle();
						break;
					case LOG_SAMPLE_PERIOD_100K:	// 10uS
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						Log_200_Cycle();
						break;
					default:
						periodFound = 1;
						replyStartSampling('O', lcb.buffSize);
						normalLog();
						break;
				}
				break;
			}
			index++;
		}
		if(!periodFound)
		{
			replyUnknown(COMMAND_REPLY_UNKNOWN_START_PERIOD);
		} else {
			replyEndSampling(lcb.buffSize);
		}
	} else {
		replyUnknown(COMMAND_REPLY_UNKNOWN_START_LENGTH);
	}
}

//----------------------------------------------------------
//		Get Log Condition Command
//	PACKET:
//		command			0x24
//		length(higher)	0x00
//		length(lower)	0x01
//		actionCode		0x01
//
void doGetLogConditonCommand()
{
	lcb.commandBuffer[1] = readUART_1();	// get length HIGH
	lcb.commandBuffer[2] = readUART_1();	// get length LOW
	if((lcb.commandBuffer[1] * 256 + lcb.commandBuffer[2]) == 1)
	{
		lcb.commandBuffer[3] = readUART_1();	// get action
		if(lcb.commandBuffer[3] == 0x01)
		{
			replyLogCondition();
		} else {
			replyUnknown(COMMAND_REPLY_UNKNOWN_LOGCOND_CODE);
		}
	} else {
		replyUnknown(COMMAND_REPLY_UNKNOWN_LOGCOND_LENGTH);
	}
}

//----------------------------------------------------------
//		GetData Command
//	PACKET:
//		command			0x26
//		length(higher)	0x00
//		length(lower)	0x05
//		actionCode		0x01
//		startLocation	0x?? 0x??
//		req. length		0x?? 0x??
//
void doGetLogDataCommand()
{
	lcb.commandBuffer[1] = readUART_1();	// get length HIGH
	lcb.commandBuffer[2] = readUART_1();	// get length LOW
	if((lcb.commandBuffer[1] * 256 + lcb.commandBuffer[2]) == 5)
	{
		lcb.commandBuffer[3] = readUART_1();	// get action
		if(lcb.commandBuffer[3] == 0x01)
		{
			lcb.commandBuffer[4] = readUART_1();	// get action
			lcb.commandBuffer[5] = readUART_1();	// get action
			lcb.commandBuffer[6] = readUART_1();	// get action
			lcb.commandBuffer[7] = readUART_1();	// get action
			replyLogData(lcb.commandBuffer[4] * 256 + lcb.commandBuffer[5],
						 lcb.commandBuffer[6] * 256 + lcb.commandBuffer[7]);
		} else {
			replyUnknown(COMMAND_REPLY_UNKNOWN_GETDATA_CODE);
		}
	} else {
		replyUnknown(COMMAND_REPLY_UNKNOWN_GETDATA_LENGTH);
	}
}

