/*

					Logic Scope AVR

			Copyrights 2009,  Yoshi Hiroshi


*/

#include <avr/io.h>
#include "logicScope.h"
#include "uart.h"

//**********************************************************
//		Init UART
//**********************************************************
//
//----------------------------------------------------------
void initUART()
{
	switch(BAUD_RATE_OPTION_PIN & BAUD_RATE_OPTION_MASK)
	{
#if defined (__AVR_ATmega644P__)
	case 0:		// 4.8K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 2;
		UART_BAUD_RATE_REGISTER_L = 8;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 1:		// 9.6K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 1;
		UART_BAUD_RATE_REGISTER_L = 3;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 2:		// 14.4K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 173;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 3:		// 19.2K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 129;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 4:		// 38.4K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 64;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 5:		// 57.6K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 42;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 6:		// 115.2K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 21;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 7:		// 230.4K at 20MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 10;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
#endif
#if defined (__AVR_ATmega128__)
	case 0:		// 4.8K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 416 / 256;
		UART_BAUD_RATE_REGISTER_L = 416 % 256;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 1:		// 9.6K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 207;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 2:		// 14.4K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 138;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 3:		// 19.2K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 103;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 4:		// 38.4K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 51;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 5:		// 57.6K at 16MHz
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 34;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
	case 6:		// 115.2K at 16MHz
	case 7:
		UART_BAUD_RATE_REGISTER_H = 0;
		UART_BAUD_RATE_REGISTER_L = 16;
		UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
		break;
#endif
	}
	UART_TRL_STATUS_C = UART_CHARACTER_SIZE;
	UART_TRL_STATUS_B = _BV(UART_TRANSMITTER_ENABLE_BIT) | _BV(UART_RECIEVER_ENABLE_BIT);
	UART_TRANSMITTER_PORT_DIRECTION_SET;
}

//**********************************************************
//		Recieve Data from UART
//**********************************************************
//
//----------------------------------------------------------
unsigned char readUART(unsigned char flag)
{
	if(!flag)
	{
		if((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) == 0)
			return 0;
	}
	unsigned char val;
	while((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) == 0);
	val = UART_DATA_REGISTER;
//	UDR1 = val;
	return val;
}

unsigned char checkReceiveUART()
{
	return ((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) != 0);
}

//**********************************************************
//		Send Data via UART
//**********************************************************
//
//----------------------------------------------------------
void sendUART(unsigned char data)
{
	while((UART_TRL_STATUS_A & _BV(UART_DATA_EMPTY_BIT)) == 0);
	UART_DATA_REGISTER = data;
}

void inhibitReceiveUART(unsigned char flag)
{
}

//**********************************************************
//		Recieve Data from UART (Hex Format)
//**********************************************************
//----------------------------------------------------------
unsigned char readUART_1()
{
	return readUART(UART_READ_WAIT_COMPLETION);
}


